@@ -39,6 +39,7 @@
  */
 #include "ogr_fdw.h"
 
+
 PG_MODULE_MAGIC;
 
 /*
@@ -183,7 +184,7 @@ Oid GEOMETRYOID = InvalidOid;
 
 #if GDAL_VERSION_NUM >= GDAL_COMPUTE_VERSION(2,1,0)
 
-const char* const gdalErrorTypes[] =
+static const char* const gdalErrorTypes[] =
 {
 	"None",
 	"AppDefined",
@@ -1034,6 +1035,9 @@ ogrGetForeignPaths(PlannerInfo* root,
 	                 NULL, /* PathTarget */
 #endif
 	                 baserel->rows,
+#if PG_VERSION_NUM >= 180000
+	                 0,       /* disabled_nodes */
+#endif
 	                 planstate->startup_cost,
 	                 planstate->total_cost,
 	                 NIL,     /* no pathkeys */
@@ -1049,7 +1053,6 @@ ogrGetForeignPaths(PlannerInfo* root,
 	        );   /* no fdw_private data */
 }
 
-
 /*
  * Convert an OgrFdwSpatialFilter into a List so it can
  * be safely passed through the fdw_private list.
@@ -1466,11 +1469,8 @@ ogrReadColumnData(OgrFdwState* state)
 		OgrFieldEntry* found_entry;
 		OgrFieldEntry entry;
 
-#if PG_VERSION_NUM >= 110000
-		Form_pg_attribute att_tuple = &tupdesc->attrs[i];
-#else
-		Form_pg_attribute att_tuple = tupdesc->attrs[i];
-#endif
+		Form_pg_attribute att_tuple = TupleDescAttr(tupdesc, i);
+
 		OgrFdwColumn col = tbl->cols[i];
 		col.pgattnum = att_tuple->attnum;
 		col.pgtype = att_tuple->atttypid;
@@ -1602,60 +1602,77 @@ ogrReadColumnData(OgrFdwState* state)
 	return;
 }
 
+
 /*
  * ogrLookupGeometryFunctionOid
  *
  * Find the procedure Oids of useful functions so we can call
- * them later.
+ * them later. In the case where multiple functions have the
+ * same signature, in different namespaces (???) we might have
+ * problems, but that seems very unlikely.
+ *
+ * https://github.com/pramsey/pgsql-ogr-fdw/issues/266
  */
 static Oid
 ogrLookupGeometryFunctionOid(const char* proname)
 {
-	List* names;
-	FuncCandidateList clist;
+	CatCList   *clist;
+	int			i;
+	Oid result = InvalidOid;
 
 	/* This only works if PostGIS is installed */
-	if (ogrGetGeometryOid() == InvalidOid || ogrGetGeometryOid() == BYTEAOID)
+	if (ogrGetGeometryOid() == InvalidOid ||
+	    ogrGetGeometryOid() == BYTEAOID)
 	{
-		return InvalidOid;
+		return result;
 	}
 
-#if PG_VERSION_NUM < 160000
-	names = stringToQualifiedNameList(proname);
-#else
-	names = stringToQualifiedNameList(proname,NULL);
-#endif
-#if PG_VERSION_NUM < 90400
-	clist = FuncnameGetCandidates(names, -1, NIL, false, false);
-#elif PG_VERSION_NUM < 140000
-	clist = FuncnameGetCandidates(names, -1, NIL, false, false, false);
-#else
-	clist = FuncnameGetCandidates(names, -1, NIL, false, false, false, false);
-#endif
+	if (!proname) return result;
+
+	/* Search syscache by name only */
+	clist = SearchSysCacheList1(PROCNAMEARGSNSP,
+	                            CStringGetDatum(proname));
 
 	if (!clist) return InvalidOid;
 
-	if (streq(proname, "st_setsrid"))
+	for (i = 0; i < clist->n_members; i++)
 	{
-		do
+		HeapTuple    proctup = &clist->members[i]->tuple;
+		Form_pg_proc procform = (Form_pg_proc) GETSTRUCT(proctup);
+		Oid         *proargtypes = procform->proargtypes.values;
+		int          pronargs = procform->pronargs;
+
+#if PG_VERSION_NUM >= 120000
+		Oid procoid = procform->oid;
+#else
+		Oid procoid = HeapTupleGetOid(proctup);
+#endif
+
+		/* ST_SetSRID(geometry, integer) */
+		if (streq(proname, "st_setsrid") &&
+		    pronargs == 2 &&
+		    proargtypes[0] == ogrGetGeometryOid())
 		{
-			int i;
-			for (i = 0; i < clist->nargs; i++)
-			{
-				if (clist->args[i] == ogrGetGeometryOid())
-				{
-					return clist->oid;
-				}
-			}
+			result = procoid;
+			break;
+		}
+		/* postgis_typmod_srid(typmod) */
+		else if (streq(proname, "postgis_typmod_srid") &&
+		         pronargs == 1)
+		{
+			/* postgis_typmod_srid(integer) */
+			result = procoid;
+			break;
+		}
+		else
+		{
+			elog(ERROR, "%s could not find function '%s'",
+			     __func__, proname);
 		}
-		while ((clist = clist->next));
-	}
-	else if (streq(proname, "postgis_typmod_srid"))
-	{
-		return clist->oid;
 	}
 
-	return InvalidOid;
+	ReleaseSysCacheList(clist);
+	return result;
 }
 
 /*
@@ -2067,7 +2084,7 @@ ogrFeatureToSlot(const OGRFeatureH feat, TupleTableSlot* slot, const OgrFdwExecS
 				for (uint32 i = 0; i < ilist_size; i++)
 				{
 					bool is_null = false;
-					snprintf(cstr, CSTR_SZ, "%ld", ilist[i]);
+					snprintf(cstr, CSTR_SZ, OGR_FDW_FRMT_INT64, OGR_FDW_CAST_INT64(ilist[i]));
 					abs = accumArrayResult(abs,
 					          pgDatumFromCString(cstr, &col, execstate->ogr.char_encoding, &is_null),
 					          is_null,
@@ -2675,13 +2692,10 @@ ogrGetFidColumn(const TupleDesc td)
 	int i;
 	for (i = 0; i < td->natts; i++)
 	{
-#if PG_VERSION_NUM >= 110000
-		NameData attname = td->attrs[i].attname;
-		Oid atttypeid = td->attrs[i].atttypid;
-#else
-		NameData attname = td->attrs[i]->attname;
-		Oid atttypeid = td->attrs[i]->atttypid;
-#endif
+		Form_pg_attribute att_tuple = TupleDescAttr(td, i);
+		NameData attname = att_tuple->attname;
+		Oid atttypeid = att_tuple->atttypid;
+
 		if ((atttypeid == INT4OID || atttypeid == INT8OID) &&
 		        strcaseeq("fid", attname.data))
 		{
@@ -2721,7 +2735,8 @@ ogrAddForeignUpdateTargets(PlannerInfo* planinfo,
 		elog(ERROR, "table '%s' does not have a 'fid' column", RelationGetRelationName(target_relation));
 	}
 
-	att = &tupdesc->attrs[fid_column];
+	att = TupleDescAttr(tupdesc, fid_column);
+
 	/* Make a Var representing the desired value */
 	var = makeVar(parsetree->resultRelation,
 	              att->attnum,
@@ -2833,6 +2848,7 @@ ogrExecForeignUpdate(EState* estate,
 	int64 fid;
 	OGRFeatureH feat;
 	OGRErr err;
+	Form_pg_attribute attrs;
 
 	elog(DEBUG3, "%s: entered function", __func__);
 
@@ -2849,11 +2865,9 @@ ogrExecForeignUpdate(EState* estate,
 
 	/* What is the value of the FID for this record? */
 	fid_datum = slot->tts_values[fid_column];
-#if PG_VERSION_NUM >= 110000
-	fid_type = td->attrs[fid_column].atttypid;
-#else
-	fid_type = td->attrs[fid_column]->atttypid;
-#endif
+	attrs = TupleDescAttr(td, fid_column);
+	fid_type = attrs->atttypid;
+
 	if (fid_type == INT8OID)
 	{
 		fid = DatumGetInt64(fid_datum);
@@ -3070,6 +3084,7 @@ ogrExecForeignDelete(EState* estate,
 	Datum fid_datum;
 	int64 fid;
 	OGRErr err;
+	Form_pg_attribute attrs;
 
 	elog(DEBUG3, "%s: entered function", __func__);
 
@@ -3086,11 +3101,8 @@ ogrExecForeignDelete(EState* estate,
 
 	/* What is the value of the FID for this record? */
 	fid_datum = planSlot->tts_values[fid_column];
-#if PG_VERSION_NUM >= 110000
-	fid_type = td->attrs[fid_column].atttypid;
-#else
-	fid_type = td->attrs[fid_column]->atttypid;
-#endif
+	attrs = TupleDescAttr(td, fid_column);
+	fid_type = attrs->atttypid;
 
 	if (fid_type == INT8OID)
 	{
