@@ -22,6 +22,7 @@
 #include "access/reloptions.h"
 #include "access/sysattr.h"
 #include "access/transam.h"
+#include "access/tupdesc.h"
 #include "catalog/indexing.h"
 #include "catalog/namespace.h"
 #include "catalog/pg_collation.h"
@@ -51,6 +52,7 @@
 #include "parser/parsetree.h"
 #include "storage/ipc.h"
 #include "utils/builtins.h"
+#include "utils/catcache.h"
 #include "utils/date.h"
 #include "utils/fmgroids.h"
 #include "utils/lsyscache.h"
@@ -84,6 +86,8 @@
 /* hexwkb is not. */
 #define OGR_FDW_HEXWKB TRUE
 
+extern Oid GEOMETRYOID;
+
 typedef enum
 {
 	OGR_UNMATCHED,
